@@ -3,13 +3,13 @@
 
 from collections import Counter
 from itertools import chain
+from typing import List, Optional
 
 import nalp.utils.constants as c
-import nalp.utils.loader as l
-import nalp.utils.logging as log
 from nalp.core import Corpus
+from nalp.utils import loader, logging
 
-logger = log.get_logger(__name__)
+logger = logging.get_logger(__name__)
 
 
 class SentenceCorpus(Corpus):
@@ -20,28 +20,35 @@ class SentenceCorpus(Corpus):
 
     """
 
-    def __init__(self, tokens=None, from_file=None, corpus_type='char', min_frequency=1,
-                 max_pad_length=None, sos_eos_tokens=True):
+    def __init__(
+        self,
+        tokens: Optional[List[str]] = None,
+        from_file: Optional[str] = None,
+        corpus_type: Optional[str] = "char",
+        min_frequency: Optional[int] = 1,
+        max_pad_length: Optional[int] = None,
+        sos_eos_tokens: Optional[bool] = True,
+    ) -> None:
         """Initialization method.
 
         Args:
-            tokens (list): A list of tokens.
-            from_file (str): An input file to load the sentences.
-            corpus_type (str): The desired type to tokenize the sentences. Should be `char` or `word`.
-            min_frequency (int): Minimum frequency of individual tokens.
-            max_pad_length (int): Maximum length to pad the tokens.
-            sos_eos_tokens (bool): Whether start-of-sentence and end-of-sentence tokens should be used.
+            tokens: A list of tokens.
+            from_file: An input file to load the sentences.
+            corpus_type: The desired type to tokenize the sentences. Should be `char` or `word`.
+            min_frequency: Minimum frequency of individual tokens.
+            max_pad_length: Maximum length to pad the tokens.
+            sos_eos_tokens: Whether start-of-sentence and end-of-sentence tokens should be used.
 
         """
 
-        logger.info('Overriding class: Corpus -> SentenceCorpus.')
+        logger.info("Overriding class: Corpus -> SentenceCorpus.")
 
         super(SentenceCorpus, self).__init__(min_frequency=min_frequency)
 
         # Checks if there are not pre-loaded tokens
         if not tokens:
             # Loads the sentences from file
-            sentences = l.load_txt(from_file).splitlines()
+            sentences = loader.load_txt(from_file).splitlines()
 
             # Creates a tokenizer based on desired type
             pipe = self._create_tokenizer(corpus_type)
@@ -62,16 +69,19 @@ class SentenceCorpus(Corpus):
         # Builds the vocabulary based on the tokens
         self._build()
 
-        logger.debug('Sentences: %d | Minimum frequency: %d | Maximum pad length: %s | '
-                     'Use <SOS> and <EOS>: %s | Vocabulary size: %d.',
-                     len(self.tokens), self.min_frequency, max_pad_length,
-                     sos_eos_tokens, len(self.vocab))
-        logger.info('SentenceCorpus created.')
+        logger.debug(
+            "Sentences: %d | Minimum frequency: %d | Maximum pad length: %s | "
+            "Use <SOS> and <EOS>: %s | Vocabulary size: %d.",
+            len(self.tokens),
+            self.min_frequency,
+            max_pad_length,
+            sos_eos_tokens,
+            len(self.vocab),
+        )
+        logger.info("SentenceCorpus created.")
 
-    def _check_token_frequency(self):
-        """Cuts tokens that do not meet a minimum frequency value.
-
-        """
+    def _check_token_frequency(self) -> None:
+        """Cuts tokens that do not meet a minimum frequency value."""
 
         # Calculates the frequency of tokens
         tokens_frequency = Counter(chain.from_iterable(self.tokens))
@@ -86,12 +96,12 @@ class SentenceCorpus(Corpus):
                     # Replaces with an unknown token
                     self.tokens[i][j] = c.UNK
 
-    def _pad_token(self, max_pad_length, sos_eos_tokens):
+    def _pad_token(self, max_pad_length: int, sos_eos_tokens: bool) -> None:
         """Pads the tokens into a fixed length.
 
         Args:
-            max_pad_length (int): Maximum length to pad the tokens.
-            sos_eos_tokens (bool): Whether start-of-sentence and end-of-sentence tokens should be used.
+            max_pad_length: Maximum length to pad the tokens.
+            sos_eos_tokens: Whether start-of-sentence and end-of-sentence tokens should be used.
 
         """
 
@@ -116,10 +126,8 @@ class SentenceCorpus(Corpus):
                 self.tokens[i].insert(0, c.SOS)
                 self.tokens[i].append(c.EOS)
 
-    def _build(self):
-        """Builds the vocabulary based on the tokens.
-
-        """
+    def _build(self) -> None:
+        """Builds the vocabulary based on the tokens."""
 
         # Creates the vocabulary
         self.vocab = sorted(set(chain.from_iterable(self.tokens)).union({c.UNK}))
